import warnings

import numpy as np

from regression.feature_construct import FeatureConstruction
from regression.feature_select import FeatureSelection
from utils.data_process import DataProcessing

warnings.filterwarnings('ignore')
np.set_printoptions(precision=3)


class FeatureEngineering:
    def __init__(self, pro_name='COAD', model_name='AE', max_epochs=3000, min_epochs=20, batch_size=20,
                 lr=1e-4):
        self.fs_record_path = 'record/fs_record/'
        self.fc_record_path = 'record/fc_record/'
        self.pro_name = pro_name
        self.model_name = model_name
        self.dp = DataProcessing(pro_name=pro_name, model_name=model_name)
        self.fs = FeatureSelection(pro_name=pro_name, model_name=model_name)
        self.fc = FeatureConstruction(pro_name=pro_name, model_name=model_name, min_epochs=min_epochs,
                                      max_epochs=max_epochs)
        self.max_epochs, self.min_epochs = max_epochs, min_epochs
        self.batch_size = batch_size
        self.lr = lr

    def getMaxFactor(self, num):
        count = 10
        while count > 1:
            if num % count == 0:
                return count
            count -= 1
        return count

    # 堆叠
    def stack(self):
        best_mae = 5000
        i, feature_num = 0, 0
        k1_list = np.arange(20, 100, 50)
        k2_list = np.arange(10, 50, 20)
        result_list = [['k1', 'k2', 'feature_num', 'best_mae', 'time']]
        loss_list = np.array([['loss', 'time']])
        while 1:

            if i == 0:
                k1, k2 = 1, 1
                temp_mae, fea_num = self.dp.get_first_fs()
                print('[first_select_mae]: {}'.format(temp_mae))
            else:
                matrix_t, days, train_len, m = self.dp.get_fs_inp(i)
                if matrix_t.shape[1] == 0:
                    print('[feature_num]: {}\t[best_mae]: {}\t[time]: {}'.format(feature_num, best_mae, i - 1))
                    with open(self.fs_record_path + self.model_name + '_' + self.pro_name + '.txt', 'w+') as file:
                        for item in result_list:
                            for a in item:
                                file.write(str(a) + '\t')
                            file.write('\n')
                    with open(self.fc_record_path + self.model_name + '_' + self.pro_name + '.txt', 'w+') as loss_f:
                        for i in range(loss_list.shape[0]):
                            for j in range(loss_list.shape[1]):
                                loss_f.write(str(loss_list[i][j]) + '\t')
                            loss_f.write('\n')
                k1, k2, temp_mae = self.fs.get_k1k2_result(matrix_t, days, m, k1_list, k2_list,
                                                           n_splits=5)
                temp_mae, fea_num = self.fs.inter_fs(i, matrix_t, days, m, k1, k2)
                if (fea_num <= 45) and (temp_mae > best_mae) and (i <= 2):
                    best_mae = 5000
                    i, feature_num = 0, 0
                    k1_list = np.arange(20, 100, 50)
                    k2_list = np.arange(10, 50, 20)
                    result_list = [['k1', 'k2', 'feature_num', 'best_mae', 'time']]
                    loss_list = np.array([['loss', 'time']])
                    continue

            if temp_mae > best_mae:
                if i < 2:
                    best_mae = 5000
                    i, feature_num = 0, 0
                    k1_list = np.arange(20, 100, 50)
                    k2_list = np.arange(10, 50, 20)
                    result_list = [['k1', 'k2', 'feature_num', 'best_mae', 'time']]
                    loss_list = np.array([['loss', 'time']])
                    continue
                print('[feature_num]: {}\t[best_mae]: {}\t[time]: {}'.format(feature_num, best_mae, i - 1))
                with open(self.fs_record_path + self.model_name + '_' + self.pro_name + '.txt', 'w+') as file:
                    for item in result_list:
                        for a in item:
                            file.write(str(a) + '\t')
                        file.write('\n')
                with open(self.fc_record_path + self.model_name + '_' + self.pro_name + '.txt', 'w+') as loss_f:
                    for i in range(loss_list.shape[0]):
                        for j in range(loss_list.shape[1]):
                            loss_f.write(str(loss_list[i][j]) + '\t')
                        loss_f.write('\n')

                return i-1
            else:
                best_mae = temp_mae
                feature_num = fea_num
                num_heads = self.getMaxFactor(fea_num)
                if num_heads <= 1:
                    num_heads = 1
                n1, n2 = fea_num, int((fea_num / 5) / num_heads) * num_heads
                gradient = int((fea_num / 10) / num_heads) * num_heads
                if n1 <= 1:
                    n1 = 1
                if n2 <= 1:
                    n2 = 1
                    num_heads = 1
                if gradient >= n2:
                    gradient = 0
                self.fc.max_epochs, self.fc.min_epochs, self.fc.batch_size, self.fc.lr, self.fc.n1, self.fc.n2, self.fc.num_heads, self.fc.gradient = \
                    self.max_epochs, self.min_epochs, self.batch_size, self.lr, n1, n2, num_heads, gradient
                k1_list, k2_list = np.arange(10, 2 * n1 + 2 * n2, 20), np.arange(5, 2 * n1 + 2 * n2, 10)
                loss_temp = self.fc.construct(i)
                loss_list = np.vstack((loss_list, loss_temp))
                i += 1
                result_list.append([k1, k2, feature_num, best_mae, i])
