import argparse
import warnings

from utils.data_preprocess import DataPreprocessing
from regression.evaluate import Evaluation
from regression.feature_engineer import FeatureEngineering
from regression.feature_select import FeatureSelection
from classification.interface import Interface

warnings.filterwarnings('ignore')


def run():
    parser = argparse.ArgumentParser()
    parser.add_argument('--pro_name', '-pn', required=False,
                        choices=['BLCA', 'COAD', 'UCEC', 'LGG', 'CESC', 'BRCA', 'LUAD', 'STAD', 'LIHC', 'SARC'],
                        type=str)
    parser.add_argument('--operation', '-o', required=True,
                        choices=['preprocess','select_classify','stack', 'evaluate', 'first_select','all_steps'], type=str)
    parser.add_argument('--model_name', '-mn', required=False,
                        choices=(
                        ['VAE', 'SAE', 'AE', 'AE_WGAN', 'SAE_WGAN', 'VAE_WGAN', 'MA_MyModel', 'MyModel', 'MA_SAE_SGAN',
                         'MA_S_AE_WGAN', 'MA_SAE_WGAN']))
    parser.add_argument('--time', '-i', required=False, type=str)
    parser.add_argument('--min_epochs', '-mine', required=False, type=str)
    parser.add_argument('--max_epochs', '-maxe', required=False, type=str)
    args = parser.parse_args()
    # 必须设置的参数
    operation = args.operation

    if operation=='preprocess':
        print('======preprocess=====')
        dpp=DataPreprocessing()
        pro_lists=[['BLCA', 'COAD', 'UCEC',  'LGG', 'CESC', 'BRCA'],['SARC', 'STAD'],['LIHC', 'LUAD']]
        year_list=[5,3,1]
        for i in range(len(year_list)):
            dpp.preprocess(pro_lists[i],year_list[i])

    if operation == 'select_classify':
        print('======select_classify=====')
        pro_name = args.pro_name
        interface = Interface(pro_name)
        interface.select_all()

    if operation == 'first_select':
        print('======first_select=====')
        pro_name = args.pro_name
        fs = FeatureSelection(pro_name=pro_name)
        fs.get_first_time()

    if operation == 'stack':
        print('======stack=====')
        pro_name = args.pro_name
        model_name = args.model_name
        max_epochs = int(args.max_epochs)
        min_epochs = int(args.min_epochs)
        fe = FeatureEngineering(pro_name=pro_name, model_name=model_name, max_epochs=max_epochs, min_epochs=min_epochs)
        fe.stack()

    if operation == 'evaluate':
        print('======evaluate=====')
        pro_name = args.pro_name
        model_name = args.model_name
        e = Evaluation(pro_name=pro_name, model_name=model_name)
        time = args.time
        e.getEvaluationResults(int(time))

    if operation=='all_steps':
        print('======preprocess=====')
        dpp = DataPreprocessing()
        pro_lists = [['BLCA', 'COAD', 'UCEC', 'LGG', 'CESC', 'BRCA'], ['SARC', 'STAD'], ['LIHC', 'LUAD']]
        year_list = [5, 3, 1]
        for i in range(len(year_list)):
            dpp.preprocess(pro_lists[i], year_list[i])
        pro_list=['BLCA', 'COAD', 'UCEC', 'LGG', 'CESC', 'BRCA', 'LUAD', 'STAD', 'LIHC', 'SARC']
        for pro_name in pro_list:
            interface = Interface(pro_name)
            interface.select_all()

            fs = FeatureSelection(pro_name=pro_name)
            fs.get_first_time()

            fe = FeatureEngineering(pro_name=pro_name, model_name='MA_MyModel', max_epochs=5000,
                                    min_epochs=100)
            t=fe.stack()

            e = Evaluation(pro_name=pro_name, model_name='MA_MyModel')
            e.getEvaluationResults(int(t))


if __name__ == '__main__':
    run()
